# AUTOGENERATED! DO NOT EDIT! File to edit: dictUtil.ipynb (unless otherwise specified).

__all__ = ['printDict', 'allKeysInDict', 'filterDt', 'stripDict', 'hashDict', 'saveDictToFile', 'loadDictFromFile',
           'saveStringToFile', 'loadStringFromFile', 'genSchema', 'printYaml']

# Cell
import yaml
from beartype import beartype

# Cell
def printDict(d:dict, length:int = 10, space = 0):
  '''print dictionary as first x value of values'''
  if type(d) != dict:
    print('this is not a dict')
    print(d)
  else:
    for k, v in d.items():
      if type(v) == dict:
        print(f"{' '*space}{k}")
        printDict(v, space=space+1)
      else:
        print(f"{' '*space}{k} : {v[:length] if type(v)==str else v}")

# Cell
def allKeysInDict(inputDict:dict, keys:list):
  return all(key in inputDict for key in keys)

# Cell
def filterDt(dtDict:dict):
  '''convert unjsonable datetime object to timestamp in the dictionary'''
  from datetime import datetime
  return {k: (filterDt(v) if type(v) == dict else v) if type(v) != datetime else v.timestamp()
            for k,v in dtDict.items()}

# Cell
def stripDict(data:dict):
  return {k: v.strip() if type(v) == str else v for k,v in data.items()}

# Cell
import hashlib, pickle, base64
def hashDict(data:dict, hasher= hashlib.sha1(), encoder = pickle.dumps):
  hasher.update(encoder(data))
  rawHash = hasher.digest()
  return base64.b64encode(rawHash).decode()

# Cell
import pickle
def saveDictToFile(data:dict, path:str):
  with open(path, 'wb')as f:
    pickle.dump(data,f,protocol=pickle.HIGHEST_PROTOCOL)

def loadDictFromFile(path:str):
  with open(path, 'rb') as f:
    return pickle.load(f)

# Cell
def saveStringToFile(data:str, path:str):
  with open(path, 'w')as f:
    f.write(data)
def loadStringFromFile(path:str):
  with open(path, 'r')as f:
    return f.read()

# Cell
def genSchema(inputDict:dict, format_='yaml')->(dict,str):
  '''generate a json schema from dict,
  format::str:
    default='yaml', return schema in json or yaml
    'json', return the json schema
  response
    'both', return a tuple of (json, yaml)
    dict or string
  '''
  from genson import SchemaBuilder
  import yaml
  builder = SchemaBuilder()
  builder.add_object(inputDict)
  schema = builder.to_schema()
  if format_=='yaml':
    return yaml.dump(schema)
  elif format_ == 'json':
    return schema
  elif format_ == 'both':
    return schema, yaml.dump(schema)
  else:
    return schema, yaml.dump(schema)

# Cell
@beartype
def printYaml(input_:dict, returnYaml = False)->(str, None):
  yamlStr:str =  yaml.dump(input_)
  print(yamlStr)
  if returnYaml:
    return yamlStr


# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"imageFromS3": "Images.ipynb",
         "imageFromUrl": "Images.ipynb",
         "imageToS3": "Images.ipynb",
         "resizeImage": "Images.ipynb",
         "showImgS3": "Images.ipynb",
         "async_wrap": "async.ipynb",
         "asyncMap": "async.ipynb",
         "asyncAwaitMap": "async.ipynb",
         "printDict": "dictUtil.ipynb",
         "allKeysInDict": "dictUtil.ipynb",
         "filterDt": "dictUtil.ipynb",
         "stripDict": "dictUtil.ipynb",
         "hashDict": "dictUtil.ipynb",
         "saveDictToFile": "dictUtil.ipynb",
         "loadDictFromFile": "dictUtil.ipynb",
         "saveStringToFile": "dictUtil.ipynb",
         "loadStringFromFile": "dictUtil.ipynb",
         "genSchema": "dictUtil.ipynb",
         "printYaml": "dictUtil.ipynb",
         "errorString": "exception.ipynb",
         "getDfHash": "pdUtils.ipynb",
         "saveLocalCache": "pdUtils.ipynb",
         "saveLocalHash": "pdUtils.ipynb",
         "loadLocalCache": "pdUtils.ipynb",
         "loadLocalHash": "pdUtils.ipynb",
         "saveRemoteHash": "pdUtils.ipynb",
         "saveRemoteCache": "pdUtils.ipynb",
         "loadRemoteCache": "pdUtils.ipynb",
         "loadRemoteHash": "pdUtils.ipynb",
         "PandasDataFrameAttribute": "pdUtils.ipynb",
         "forceType": "pdUtils.ipynb",
         "SchemaAttribute": "pynamodbAttributes.ipynb",
         "extraDocsString": "pynamodbAttributes.ipynb",
         "SuperModel": "pynamodbAttributes.ipynb",
         "createData": "pynamodbAttributes.ipynb",
         "getData": "pynamodbAttributes.ipynb",
         "updateData": "pynamodbAttributes.ipynb",
         "getSchemaPath": "schema.ipynb",
         "validateUrl": "schema.ipynb",
         "getTypes": "schema.ipynb",
         "typeMapJsonSchema": "schema.ipynb",
         "typeMap": "schema.ipynb",
         "getSecret": "secrets.ipynb",
         "shorten": "shortenLInk.ipynb",
         "url": "shortenLInk.ipynb",
         "Timer": "timer.ipynb",
         "start_timer": "timer.ipynb",
         "reset_timer": "timer.ipynb",
         "print_time": "timer.ipynb",
         "print_reset": "timer.ipynb",
         "add_method": "wrappers.ipynb",
         "add_static_method": "wrappers.ipynb",
         "add_class_method": "wrappers.ipynb"}

modules = ["images.py",
           "asyncUtil.py",
           "dictUtil.py",
           "exception.py",
           "pdUtils.py",
           "pynamodb.py",
           "schema.py",
           "secrets.py",
           "shortenLink.py",
           "timer.py",
           "wrappers.py"]

doc_url = "https://thanakijwanavit.github.io/nicHelper/"

git_url = "https://github.com/thanakijwanavit/nicHelper/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: async.ipynb (unless otherwise specified).

__all__ = ['async_wrap', 'asyncMap', 'asyncAwaitMap']

# Cell
import asyncio
from functools import wraps, partial

def async_wrap(func):
    @wraps(func)
    async def run(*args, loop=None, executor=None, **kwargs):
        if loop is None:
            loop = asyncio.get_event_loop()
        pfunc = partial(func, *args, **kwargs)
        return await loop.run_in_executor(executor, pfunc)
    return run


# Cell
import multiprocessing.dummy
from typing import Callable, List, Any, Iterable
from beartype import beartype
@beartype
def asyncMap(f:Callable, data:Iterable[Any], threads:int = 5)->Any:
  p = multiprocessing.dummy.Pool(threads)
  return p.map(f,data)

# Cell
def asyncAwaitMap(f:Callable, data:Iterable[Any])->Any:
  af = async_wrap(f) # convert to async func
  async def runLoop():
    rtup = (af(i) for i in data)
    return await asyncio.gather(*rtup)
  return asyncio.run(runLoop())


# AUTOGENERATED! DO NOT EDIT! File to edit: wrappers.ipynb (unless otherwise specified).

__all__ = ['add_method', 'add_static_method', 'add_class_method']

# Cell
from functools import wraps # This convenience func preserves name and docstring


# Cell
def add_method(cls):
    def decorator(func):
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            return func(self, *args, **kwargs)
        setattr(cls, func.__name__, wrapper)
        # Note we are not binding func, but wrapper which accepts self but does exactly the same as func
        return func # returning func means func can still be used normally
    return decorator


# Cell
def add_static_method(cls):
  def decorator(func):
    @wraps(func)
    def wrapper( *args, **kwargs):
      return func(*args, **kwargs)
    setattr(cls, func.__name__, wrapper)
  return decorator

# Cell
def add_class_method(cls):
    def decorator(func):
        @wraps(func)
        @classmethod
        def wrapper(cls, *args, **kwargs):
            return func( cls, *args, **kwargs)
        setattr(cls, func.__name__, wrapper)
        # Note we are not binding func, but wrapper which accepts self but does exactly the same as func
        return func # returning func means func can still be used normally
    return decorator


# AUTOGENERATED! DO NOT EDIT! File to edit: schema.ipynb (unless otherwise specified).

__all__ = ['getSchemaPath', 'validateUrl', 'getTypes', 'typeMapJsonSchema', 'typeMap']

# Cell
import jsonschema, requests, yaml
from types import SimpleNamespace

# Cell
import dpath.util
def getSchemaPath(schemaUrl:str, path:str='/', isYaml = True):
  '''
  get a nested schema from path
  schemaUrl: str: url of the schema
  path: str: path of the schema, if root then path='/'
  isYaml: Bool: is the schema yaml (false indicates that the schema is json)
  '''
  if isYaml:
    schema = yaml.load(requests.get(schemaUrl).text, Loader= yaml.Loader)
  else:
    schema = requests.get(schemaUrl).json()
  return dpath.util.get(schema, path)

# Cell
def validateUrl(url,input_, format_ = 'json', headers = {'Cache-Control': 'no-cache'}, path = '/'):
  if format_ == 'yaml':
    schema = getSchemaPath(url, path = path, isYaml = True)
  elif format_ == 'json':
    schema = getSchemaPath(url, path = path, isYaml = False)
  else:
    print('invalid schema format, using json')
    schema = requests.get(url).json()
  res = jsonschema.validate(input_,schema)
  return SimpleNamespace(**input_)

# Cell
typeMap = {'string': str, 'number': float, 'integer': int, 'object': dict, 'array': list, 'boolean': bool, 'null': None}
def getTypes(schemaUrl:str, typeMap:dict=typeMap)->dict:
  '''get python types from json schema'''
  r = requests.get(schemaUrl)
  s = yaml.load(r.text, Loader=yaml.FullLoader)
  properties = s['properties']
  dtypes = {k: typeMap.get(v['type']) for k,v in properties.items()}
  return dtypes
def typeMapJsonSchema(url:str, input_:dict = {}, typeMap:dict = typeMap, defaultType=str):
  '''
  try to map the datatype into the one specified in url of json schema.
  if type is not found, the defaultType is used
  '''
  typesDict = getTypes(url, typeMap=typeMap) # get dtype from schema url
  print(f'typesDict is: {typesDict}')
  convertedInput = {k: (typesDict.get(k) or defaultType)(v) for k,v in input_.items()}
  return convertedInput




# AUTOGENERATED! DO NOT EDIT! File to edit: exception.ipynb (unless otherwise specified).

__all__ = ['errorString']

# Cell
import traceback

# Cell
def errorString():
  return traceback.format_exc()


# AUTOGENERATED! DO NOT EDIT! File to edit: timer.ipynb (unless otherwise specified).

__all__ = ['Timer', 'start_timer', 'reset_timer', 'print_time', 'print_reset']

# Cell
from .wrappers import add_class_method,add_method,add_static_method
from datetime import datetime, timedelta

# Cell
class Timer:
  def __init__(self):
    self.start_timer()
  pass


# Cell
@add_method(Timer)
def start_timer(self):
  self.t0 = datetime.now()
@add_method(Timer)
def reset_timer(self):
  self.t0 = datetime.now()

# Cell
@add_method(Timer)
def print_time(self, description = 'fuction took'):
  t1:timedelta = datetime.now() - self.t0
  print(f'{description} :{t1.total_seconds()} s')
  return t1.total_seconds()

# Cell
@add_method(Timer)
def print_reset(self, description = 'function took'):
  '''
  print_reset(self, description = 'function took')
  print and reset timer
  description: str: description of the item to print
  '''

  r = self.print_time(description = description)
  self.reset_timer()
  return r